module Tosuto
  class DiningOption < Resource
    ENDPOINT = '/config/v2/diningOptions'.freeze
    DSP_MATCH = %w[delivery grub uber door postmates].freeze
    CARRYOUT_MATCH = %w[carryout].freeze

    attr_accessor :guid, :entity_type, :external_id, :name, :behavior

    def initialize(hash)
      set_attributes(hash)
    end

    def get_details(token:, restaurant_id:, api: API.new)
      url = "#{ENDPOINT}/#{guid}"
      response = api.request(
        :get,
        url,
        restaurant_id: restaurant_id,
        token: token,
      )
      raise response.error unless response.success?

      set_attributes(response.json_body)
      self
    end

    def dsp?
      DSP_MATCH.any? { |dsp| name&.downcase&.match?(dsp) }
    end

    def carryout?
      CARRYOUT_MATCH.any? { |carryout| name&.downcase&.match?(carryout) }
    end

    def take_out?
      behavior == 'TAKE_OUT' && (carryout? || !dsp?)
    end

    def delivery?
      behavior == 'DELIVERY' || (dsp? && !carryout?)
    end
  end
end
