module Tosuto
  class OauthToken < Resource
    ENDPOINT = "/authentication/v1/authentication/login".freeze

    attr_accessor :access_token, :token_type, :expires_in, :expires_at, :scope

    def self.create(client_id, client_secret, user_access_type: "TOAST_MACHINE_CLIENT", api: API.new)
      body = {
        userAccessType: user_access_type,
        clientId: client_id,
        clientSecret: client_secret,
      }
      response = api.request(:post, ENDPOINT, body: body)
      raise response.error unless response.success?

      return new(response.json_body)
    end

    def initialize(attributes = {})
      return unless (token = attributes['token'])

      self.access_token = token['accessToken']
      self.expires_in = token['expiresIn'].to_i
      self.expires_at = Time.now + expires_in
      self.token_type = token['tokenType']
      self.scope = token['scope']
    end

    def expired?
      expires_at < Time.now
    end

    def inspect
      "#<#{self.class} #{access_token&.inspect}>"
    end
  end
end
