require 'date'

module Tosuto
  class Order < Resource
    ENDPOINT = "/orders/v2/orders"

    attr_accessor :guid, :external_id, :revenue_center, :server, :source, :void_date,
      :duration, :business_date, :paid_date, :restaurant_service, :voided,
      :estimated_fulfillment_date, :table, :required_prep_time, :approval_status,
      :delivery_info, :service_area, :number_of_guests, :opened_date,
      :void_business_date, :deleted, :closed_date, :deleted_date, :modified_date,
      :promised_date, :pricing_features, :restaurant_id

    attr_collections checks: Check
    attr_objects dining_option: DiningOption

    def self.all(restaurant_id:, token:, date_range: nil, business_date: nil, api: API.new)
      params = {}
      unless date_range.nil?
        params["startDate"] = date_range.begin.utc.strftime("%Y-%m-%dT%H:%M:%S.%L%z")
        params["endDate"] = date_range.end.utc.strftime("%Y-%m-%dT%H:%M:%S.%L%z")
      end
      unless business_date.nil?
        params["businessDate"] = business_date.strftime("%Y%m%d")
      end

      response = api.request(
        :get,
        ENDPOINT,
        params: params,
        restaurant_id: restaurant_id,
        token: token
      )
      raise response.error unless response.success?

      response.json_body.map { |guid|
        Order.new(guid: guid, restaurant_id: restaurant_id)
      }
    end

    def self.get(guid:, restaurant_id:, token:, api: API.new)
      url = "#{ENDPOINT}/#{guid}"
      response = api.request(
        :get,
        url,
        restaurant_id: restaurant_id,
        token: token
      )
      raise response.error unless response.success?

      return new(response.json_body.merge(restaurant_id: restaurant_id))
    end

    def initialize(hash)
      set_attributes(hash)
    end

    def get_details(token:, api: API.new)
      Order.get(guid: guid, restaurant_id: restaurant_id, token: token, api: api)
    end
  end
end
