require 'cgi'

module Tosuto
  class Resource
    def self.encode_form_data(body)
      body.map { |key, value| "#{key}=#{CGI.escape(value)}" }.join("&")
    end

    def self.attr_objects(hash)
      hash.each do |name, klass|
        attr_reader(name)
        define_method("#{name}=") { |obj|
          instance_variable_set("@#{name}", parse_object(obj, klass))
        }
      end
    end

    def self.attr_collections(hash)
      hash.each do |name, klass|
        attr_reader(name)
        define_method("#{name}=") { |ary|
          instance_variable_set("@#{name}", parse_collection(ary, klass))
        }
      end
    end

    def underscore(value)
      return value unless /[A-Z-]/.match?(value)
      word = value.to_s.gsub(/([A-Z\d]+)([A-Z][a-z])/, '\1_\2')
      word.gsub!(/([a-z\d])([A-Z])/, '\1_\2')
      word.tr!("-", "_")
      word.downcase!
      word
    end

    def set_attribute(key, value)
      setter = "#{underscore(key)}="
      send(setter, value) if respond_to?(setter)
    end

    def set_attributes(hash)
      return unless hash.is_a?(Hash)

      hash.each do |key, value|
        set_attribute(key, value)
      end
    end

    def parse_collection(ary, klass)
      ary.map { |obj|
        parse_object(obj, klass)
      }
    end

    def parse_object(obj, klass)
      if obj.is_a?(klass)
        obj
      else
        klass.new(obj)
      end
    end

    def inspect
      "#<#{self.class} #{[@guid, @name || @display_name].join(" ")}>"
    end
  end
end
